PROGRAM cpav
!
! cpa -- Carrier Phase Analysis for VIPIR
! Reads the RIQ file and extracts information about the carrier phase of an Amplutude Modulated
! broadcast signal in the radar's reciver.
! Phase vs time (Dopper) and phase vs antenna spacing (arrival angle) analysis is performed.
!
! 
! Author: Tery Bullett,   University of Colorado CIRES
! 30 July 2014
! Linux using gfortran compiler
!
!
! License:
! This software is the intellectual property of the author.
! Generous terms of use are likely if you ask nicely.
! 
! "To give glory to God by exploring His universe with reverence"  
!
! Revision History
! Version 0.10 -  Initial version
!    The assumed structure is either the passive shuffle mode where a number of fixed frequency observations 
!    are shuffled between a sweep frequency observation, or a multiple fixed frequency mode where there are no
!    sweep frequency data present.  
! 
USE misc_utils
IMPLICIT NONE

INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI
!
! Parameters
INTEGER, PARAMETER :: MAXPRI=2048
COMPLEX, PARAMETER :: CZERO=(0.0, 0.0)
REAL,PARAMETER :: VERSION=2.07
REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: DEGperRAD = 180.0/PI
!
CHARACTER(LEN=120):: INFILE
! For command line parsing
CHARACTER(LEN=120) :: CLP
INTEGER :: K,NA,IU,MASKRX,FLIPHASE,NUMSHUFFLE,PICKSHUFFLE
LOGICAL :: EOF,SKIP,VERBOSE,RAW,SHUFFLEMODE,DEBUG
CHARACTER(LEN=5) :: URSICODE
!
! Structures
TYPE(PRItype) :: PRI(MAXPRI)
TYPE(SCTtype) :: SCT
TYPE(PCTtype) :: PCT 
!
! Functions
LOGICAL :: READPRI, READSCT,READPCT
!
IU=1
EOF=.FALSE.
DEBUG=.FALSE.
MASKRX=-1      ! Select all antenas by default
FLIPHASE=0     ! Assume all antennas are wired properly
SHUFFLEMODE=.FALSE.
NUMSHUFFLE=0
PICKSHUFFLE=1
URSICODE=""! Read the command line for arguments
K = IARGC() 
! Filename must exist
IF (K.LT.1) THEN
   WRITE(*,*) "read_riq-> ERROR: Input file must be specified"
   CALL EXIT(-1)
RETURN
ENDIF
!
SKIP=.FALSE.
COHERENT=.FALSE.
!
! Parse the command line.
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-v') THEN
      VERBOSE = .TRUE.
   ELSE IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: cpa {options} inputfilename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Perform Carrier Phase Analysis on VIPIR data'
      WRITE(*,*)' '
      WRITE(*,*)' -S {n} Shuffle-mode data with {n} interleaves'
   ELSE IF (CLP.EQ.'-S') THEN
      SHUFFLEMODE=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NUMSHUFFLE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-d') THEN
      DEBUG=.TRUE.
   ENDIF
ENDDO

!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(*,*) ' readriq - error - Missing input file'
   CALL EXIT(-1)
ENDIF
!
! Zero out the ionogram (PRI) structures.
DO I=1,MAXPRI
   CALL ZeroPRI(PRI(I))
ENDDO
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             G95 has this feature, and in October 2010 gfortran version 4.4.5 also does.
!             Your mileage will vary. 
!-----------------------------------------------------------------------------------------

OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
CALL FixSCTstrings(SCT)
IF(VERBOSE) CALL DumpSCT(SCT)
! Set amplitude limits
UNDERFLOW=1.0
IF (SCT%receiver%word_format .EQ.2) UNDERFLOW=0.001
!
! Get the processing time
CALL FDATE(TimeStamp)

! Set ShuffleMode based on the flags and SCT
! NOTE:  In general there will be more than one fixed frequency, and this can not be 
!        easily determined from the SCT, so the -S {numshuffle} command is needed.
! 
SHUFFLEMODE= (SHUFFLEMODE.OR.(SCT%Frequency%tune_type.EQ.4).OR.(SCT%Frequency%tune_type.EQ.5))
IF ((SHUFFLEMODE).AND.(NUMSHUFFLE.EQ.0)) THEN
   NUMSHUFFLE = 5    ! A guess
ENDIF
!
NUMFIXED=NUMSHUFFLE-1
! Modes which contain only fixed frequency data are not shuffle modes  
IF (NUMSHUFFLE.LT.1) THEN
!  ToDo:  Determine the number of fixed frequencies from the SCT
   NUMFIXED=4
ENDIF
IF (DEBUG) WRITE(*,*) 'SHUFFLEMODE= ',SHUFFLEMODE,' NUMSHUFFLE= ',NUMSHUFFLE  
!
! Determine the decode sequence
! ToDo:  Support Ramps and Repeats.

IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start Read Loop ',TimeHack, CTime
ENDIF

EOF=.FALSE.
CTime=0.0
ISHUFFLE=0
NUMPRI=0
!
! Determine the pulse set length
NPRIBLOCK=SCT%frequency%pulse_count
IF (DEBUG) WRITE(*,*) 'NPRIBLOCK= ',NPRIBLOCK
!
DO WHILE (.NOT.EOF)           ! This is the main processing loop.
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
