PROGRAM cpav
!
! cpa -- Carrier Phase Analysis for VIPIR
! Reads the RIQ file and extracts information about the carrier phase of an Amplutude Modulated
! broadcast signal in the radar's reciver.
! Phase vs time (Dopper) and phase vs antenna spacing (arrival angle) analysis is performed.
! These are placed into an object called an Intercept.
!
!  Requires:
!  Object types PCT, SCT, PRI and Intercept
!  Subroutine:
!   initintercept
!   zeropri
!   readpri
!   
!
! 
! Author: Tery Bullett,   University of Colorado CIRES
! 30 July 2014
! Linux using gfortran compiler
!
!
! License:
! This software is the intellectual property of the author.
! Generous terms of use are likely if you ask nicely.
! 
! "To give glory to God by exploring His universe with reverence"  
!
! Revision History
! Version 0.10 -  Initial version
!    The assumed structure is either the passive shuffle mode where a number of fixed frequency observations 
!    are shuffled between a sweep frequency observation, or a multiple fixed frequency mode where there are no
!    sweep frequency data present.  
! 
USE misc_utils
IMPLICIT NONE
!
! Input data
INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI
INCLUDE 'intercept.f90'   ! The intercept object
!
! Parameters
INTEGER, PARAMETER :: MAXPRI=64
INTEGER, PARAMETER :: MAXINTERCEPT=4096
INTEGER, PARAMETER :: MAXRGT=4096
COMPLEX, PARAMETER :: CZERO=(0.0, 0.0)
REAL,PARAMETER :: VERSION=2.07
REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: DEGperRAD = 180.0/PI
!
CHARACTER(LEN=120):: INFILE
! For command line parsing
CHARACTER(LEN=120) :: CLP
INTEGER :: K,NA,IU,MASKRX,FLIPHASE,NUMSHUFFLE,PICKSHUFFLE,CALMETHOD,NFRQ,NUMFIXED
LOGICAL :: EOF,SKIP,VERBOSE,SHUFFLEMODE,DEBUG,RXCAL,INFOK
REAL:: RANGE1,RANGE2
!
! Structures
TYPE(PRItype) :: PRI(MAXPRI)
TYPE(SCTtype) :: SCT
TYPE(PCTtype) :: PCT 
TYPE(InterceptType) :: Intercept(MAXINTERCEPT)
!
!
! Local Variables
INTEGER::I,RGT1,RGT2,NUMINTERCEPT,NUMPRI,IPRI,IFIXED,NPRIBLOCK,NUMRX 
REAL:: RANGE0,UNDERFLOW,CTime,RDT
CHARACTER(LEN=5) :: URSICODE
CHARACTER(LEN=32):: TimeStamp,TimeHack
LOGICAL:: SKIPPRI
INTEGER:: NX,IRX,IRG,MODEBIN,MEDIANBIN,LQBIN,UQBIN
REAL::X(MAXRGT),DF(MAXRGT),CDF(MAXRGT),VAL(MAXRGT)       
!
! Functions
LOGICAL :: READPRI, READSCT,READPCT
!
IU=1
EOF=.FALSE.
DEBUG=.FALSE.
VERBOSE=.FALSE.
MASKRX=-1      ! Select all antenas by default
FLIPHASE=0     ! Assume all antennas are wired properly
SHUFFLEMODE=.FALSE.
NUMSHUFFLE=0
PICKSHUFFLE=1
RANGE0=0.0
URSICODE=""
RXCAL=.FALSE.
CALMETHOD=0
! Read the command line for arguments
K = IARGC() 
! Filename must exist
IF (K.LT.1) THEN
   WRITE(*,*) "read_riq-> ERROR: Input file must be specified"
   CALL EXIT(-1)
RETURN
ENDIF
!
SKIP=.FALSE.
!
! Parse the command line.
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-v') THEN
      VERBOSE = .TRUE.
   ELSE IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: cpa {options} inputfilename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Perform Carrier Phase Analysis on VIPIR passive data'
      WRITE(*,*)' '
      WRITE(*,*)' -r1 Lower range for statistics. Default=30 [km]'
      WRITE(*,*)' -r2 Upper range for statistics. Default=maximum [km]'
      WRITE(*,*)' -S {n} Shuffle-mode data with {n} interleaves'
   ELSE IF (CLP.EQ.'-r1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-S') THEN
      SHUFFLEMODE=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NUMSHUFFLE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-d') THEN
      DEBUG=.TRUE.
   ELSE 
!    This is the input file name
     INFILE=CLP
   ENDIF
ENDDO
!
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start run',TimeHack,CTime 
ENDIF
!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(*,*) ' readriq - error - Missing input file'
   CALL EXIT(-1)
ENDIF
!
! Zero out the ionogram (PRI) structures.
DO IPRI=1,MAXPRI
   CALL ZeroPRI(PRI(IPRI))
ENDDO
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             G95 has this feature, and in October 2010 gfortran version 4.4.5 also does.
!             Your mileage will vary. 
!-----------------------------------------------------------------------------------------
!
OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
CALL FixSCTstrings(SCT)
IF(VERBOSE) CALL DumpSCT(SCT)
!
! Set variables based on command line parameters
!
! Set Range Gate Limits.
RGT1=10          ! To avoid Tx and Rx impulse response 
RGT2=SCT%timing%gate_count
IF(RANGE1.GT.0) THEN
   RGT1=NINT((RANGE1-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT1=MIN(MAX(1,RGT1),SCT%timing%gate_count)
ENDIF
IF (RANGE2.GT.0) THEN
   RGT2=NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=MIN(MAX(1,RGT2),SCT%timing%gate_count)
ELSE IF (RANGE2.LT.0) THEN
   RGT2=-NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=SCT%timing%gate_count-RGT2
ENDIF
!
! Set amplitude limits
UNDERFLOW=1.0
IF (SCT%receiver%word_format .EQ.2) UNDERFLOW=0.001
!
! Get the processing time
CALL FDATE(TimeStamp)

! Set ShuffleMode based on the flags and SCT
! NOTE:  In general there will be more than one fixed frequency, and this can not be 
!        easily determined from the SCT, so the -S {numshuffle} command is needed.
! 
SHUFFLEMODE= (SHUFFLEMODE.OR.(SCT%Frequency%tune_type.EQ.4).OR.(SCT%Frequency%tune_type.EQ.5))
IF ((SHUFFLEMODE).AND.(NUMSHUFFLE.EQ.0)) THEN
   NUMSHUFFLE = 5    ! A guess
ENDIF
!
NUMFIXED=NUMSHUFFLE-1
! Modes which contain only fixed frequency data are not shuffle modes. 
! They are multiple fixed frequency modes 
IF (NUMSHUFFLE.LT.1) THEN
!  ToDo:  Determine the number of fixed frequencies from the SCT
   NUMFIXED=4
ENDIF
! From this point on, the NUMFIXED will be used instead of NUMSHUFFLE.
! The shuffle mode exception will be to skip the sweep frequency PRIs
!
IF (DEBUG) WRITE(*,*) 'SHUFFLEMODE= ',SHUFFLEMODE,' NUMSHUFFLE= ',NUMSHUFFLE, ' NUMFIXED= ', NUMFIXED  
!
! Determine the decode sequence
! ToDo:  Support Ramps and Repeats.

IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start Read Loop ',TimeHack, CTime
ENDIF

EOF=.FALSE.
CTime=0.0
IFIXED=0
NUMPRI=0
NUMINTERCEPT=0
!
! Determine the pulse set length
NPRIBLOCK=SCT%frequency%pulse_count
IF (DEBUG) WRITE(*,*) 'NPRIBLOCK= ',NPRIBLOCK
!
! For the first go-around, use RTI mode (PRI-by-PRI) instead of pulset based processing.
! In almost all cases, there will be discontunities at each PRI.
DO WHILE (.NOT.EOF)           ! This is the main processing loop.
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
!
! There will be a loop here to loop over PRIs in a pulset, TBD
   SKIPPRI=.TRUE.    ! Read at least one PRI
   DO WHILE (SKIPPRI.AND.(.NOT.EOF))
      NUMPRI=NUMPRI+1
!      IPRI=NUMPRI
      IPRI=1    !  Hack
      CALL ZeroPCT(PCT)
      EOF=READPCT(IU,PCT)
      IF(EOF) CYCLE
      CALL FixPCTstrings(PCT)
      IF(VERBOSE) CALL DumpPCT(PCT)
!     Copy relevant elements from SCT and PCT to the PRI Structure
      CALL SCTPCT2PRI(SCT,PCT, PRI(IPRI))
      EOF=READPRI(IU,FLIPHASE,SCT%receiver%word_format,   PRI(IPRI))         ! Read one A-scan
      IF(EOF) CYCLE
      IFIXED = MOD((NUMPRI-1),NUMFIXED)
      SKIPPRI=((SHUFFLEMODE).AND.(IFIXED.NE.(PICKSHUFFLE-1)))
      IF (DEBUG) THEN
         WRITE(*,*) ' NUMPRI= ',NUMPRI, 'SKIPPRI= ',SKIPPRI,' IFIXED= ',IFIXED,' PICKSHUFFLE= ',PICKSHUFFLE
         WRITE(*,*) ' IPRI= ', IPRI, 'PRI%Frequency = ',PRI(IPRI) %Frequency
      ENDIF 
   ENDDO   ! Skipping PRI's 

!     Optional signal calibration
   IF (RXCAL) CALL CALIBRATE(SCT%station%ursi_id, SCT%start_epoch, CALMETHOD, MASKRX,   PRI(IPRI))
!
! Process the PRI for Doppler shift
   NUMINTERCEPT = NUMINTERCEPT + 1
   CALL InitIntercept(SCT, URSICODE, Intercept(NUMINTERCEPT))
   Intercept(NUMINTERCEPT)%Frequency=PRI(IPRI)%Frequency
!
!  All this should probabaly mode to a separate subroutine.
!
!  Fill the PRI%Amplitude and PRI%Phase arrays.  The directions are TBD
!hack   DO IPRI=1,NUMPRI
      CALL SIMPLE_INTERFEROMETER(PRI(IPRI))
!hack   ENDDO  
!
!  The basic approach here is to perform statistics on the differences of phase with time
!  and across the array.  Assumes the aliased differences are infrequent.
!
   NUMRX=Intercept(NUMINTERCEPT)%SCT%station%rx_count 
   DO IRX=1, NUMRX
!     Amplitude Statistics
      NX=0
      DO IRG=RGT1, RGT2
         NX=NX+1
         X(NX)=PRI(IPRI)%AmpdB(IRX,IRG)
      ENDDO
      CALL PDF(-20.0,120.0,141,NX,X,  MODEBIN,MEDIANBIN,LQBIN,UQBIN,VAL,DF,CDF)
!      write(*,*) NUMINTERCEPT,IRX,LQBIN,MEDIANBIN,UQBIN,VAL(LQBIN),VAL(MEDIANBIN),VAL(UQBIN)
!      IF (DEBUG) WRITE(*,'(3I5,F8.1,2F8.3)') (NUMINTERCEPT,IRX,I,VAL(I),DF(I),CDF(I),I=1,141)
      Intercept(NUMINTERCEPT)%Amplitude(IRX)=VAL(MEDIANBIN)
      Intercept(NUMINTERCEPT)%AmplitudeVar(IRX)=MAX(VAL(MEDIANBIN)-VAL(LQBIN), VAL(UQBIN)-VAL(MEDIANBIN))      
!     Phase Differece Statistics
      RDT=DEGperRAD   ! Recriporiocal Delta Time, For testing
      NX=0
      DO IRG=RGT1+1, RGT2
         NX=NX+1
         X(NX)=(PRI(IPRI)%Phase(IRX,IRG)-PRI(IPRI)%Phase(IRX,IRG-1)) * RDT
         write(*,'(I5,3F10.3)') NX, PRI(IPRI)%Phase(IRX,IRG), PRI(IPRI)%Phase(IRX,IRG-1), X(NX)
      ENDDO
!      write(*,'(I5,F8.3)') (I,X(I),I=1,NX)
      CALL PDF(-40.0,40.0,361,NX,X,  MODEBIN,MEDIANBIN,LQBIN,UQBIN,VAL,DF,CDF)
      write(*,*) NUMINTERCEPT,IRX,LQBIN,MEDIANBIN,MODEBIN,UQBIN, VAL(LQBIN),VAL(MEDIANBIN),VAL(UQBIN)
      IF (DEBUG) WRITE(*,'(3I5,F10.3,2F8.3)') (NUMINTERCEPT,IRX,I,VAL(I),DF(I),CDF(I),I=1,361)
      Intercept(NUMINTERCEPT)%Doppler(IRX)=VAL(MEDIANBIN)
      Intercept(NUMINTERCEPT)%DopplerVar(IRX)=MAX(VAL(MEDIANBIN)-VAL(LQBIN), VAL(UQBIN)-VAL(MEDIANBIN))
   ENDDO ! Receivers
   EOF=(NUMPRI.EQ.1)  !  Hack
ENDDO    ! Reading this RIQ file
 
call EXIT(1)

DO I=1,NUMINTERCEPT
   WRITE(*,101) I,Intercept(I)%Frequency,(Intercept(I)%Amplitude(IRX),Intercept(I)%AmplitudeVar(IRX),IRX=1,NUMRX)
ENDDO

101   FORMAT (I5,F10.3,8(2F8.1)) 
END
