PROGRAM riq_intercept
!
! Compute external signal intercepts from RIQ files for VIPIR 
! Reads the RIQ file and extracts information about the carrier phase of an Amplutude Modulated
! broadcast signal in the radar's reciver.
! Phase vs time (Dopper) and phase vs antenna spacing (arrival angle) analysis is performed.
! These are placed into an object called an Intercept.
!
!  Requires:
!  Object types PCT, SCT, PRI and Intercept
!  Subroutine:
!   initintercept
!   zeropri
!   readpri
!   
!
! 
! Author: Terry Bullett,   University of Colorado CIRES
! 30 July 2014
! Linux using gfortran compiler
!
!
! License:
! This software is the intellectual property of the author.
! Generous terms of use are likely if you ask nicely.
! 
! "To give glory to God by exploring His universe with reverence"  
!
! Revision History
! 22Feb15 Version 0.10 -  Initial version  
!    The assumed structure is either the passive shuffle mode where a number of fixed frequency observations 
!    are shuffled between a sweep frequency observation, or a multiple fixed frequency mode where there are no
!    sweep frequency data present.  
!    By using a 2D array of Intercept(MAXSHUFFLE,MAXINTERCEPT) objects
!      This should be done to riq_ionogram to allow one-pass processing of
!      ionograms and multiple fixed frequency data 
! 
! Globals 
USE misc_utils
IMPLICIT NONE
INCLUDE 'type-param.f90'! Global Types and parameter definitions  
INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI
INCLUDE 'intercept.f90'   ! The intercept object
INCLUDE 'stats.f90'       ! Statisitics object
!
! Local Parameters
COMPLEX, PARAMETER :: CZERO=(0.0, 0.0)
REAL,PARAMETER :: VERSION=2.08
!
CHARACTER(LEN=120):: INFILE
! For command line parsing
CHARACTER(LEN=120) :: CLP
INTEGER :: K,NA,IU,MASKRX,FLIPHASE,NUMSHUFFLE,PICKSHUFFLE,CALMETHOD,NFRQ,NUMFIXED,NUMSWEEP,N1,N2,STATMODE
LOGICAL :: EOF,SKIP,VERBOSE,SHUFFLEMODE,DEBUG,RXCAL,INFOK,BLANKA,BLANKP,BLANKF,RAW,SIMODE,SWEEP
REAL:: RANGE1,RANGE2,RMIN
!
! Structures
! THESE ARE SCREAMING TO BE ALLOCATABLE ARRAYS.  RUNNING OUT OF MEMORY
TYPE(PRItype) :: PRI(MAXPRI)
TYPE(SCTtype) :: SCT
TYPE(PCTtype) :: PCT 
TYPE(InterceptType) :: Intercept(MAXSHUFFLE,MAXINTERCEPT)
TYPE(STATtype):: PRIStats(MAXSTATS)
TYPE(STATtype):: CollectStats  !  These could be allocatable arrays but there isn't enough space to make them full size
                                        !  So we do line-by-line processing                                
!
!
! Local Variables
!     NUMICPT - Intercept Counter/number.  Typically NPRI/NSHUFFLE
!
INTEGER::I,IRNG,ISHUFFLE,RGT1,RGT2,NUMICPT,NUMPRI,NPRI,IPRI,IFIXED,NPRIBLOCK,NI,IS
INTEGER,DIMENSION(MAXINTERCEPT)::NUMINTERCEPT  !  Total nimber of intercepts per shuffle
REAL:: RANGE0,UNDERFLOW,CTime,TK
CHARACTER(LEN=5) :: URSICODE
CHARACTER(LEN=32):: TimeStamp,TimeHack
LOGICAL:: SKIPPRI
!
! Functions
LOGICAL :: READPRI, READSCT,READPCT
!
IU=1
EOF=.FALSE.
DEBUG=.FALSE.
BLANKA=.FALSE.
BLANKP=.FALSE.
BLANKF=.FALSE.
SIMODE=.TRUE.
VERBOSE=.FALSE.
RAW=.FALSE.
SWEEP=.TRUE.   ! Does this have sweep data?
MASKRX=-1      ! Select all antenas by default
FLIPHASE=0     ! Assume all antennas are wired properly
SHUFFLEMODE=.FALSE.
NUMSHUFFLE=0
NUMSWEEP=1
PICKSHUFFLE=1
RANGE0=0.0
URSICODE=""
RXCAL=.FALSE.
NUMINTERCEPT=0
CALMETHOD=0
RMIN=0.70  ! Minimum correlation coefficient to accept
! Read the command line for arguments
K = IARGC() 
! Filename must exist
IF (K.LT.1) THEN
   WRITE(*,*) "read_riq-> ERROR: Input file must be specified"
   CALL EXIT(-1)
RETURN
ENDIF
!
SKIP=.FALSE.
!
! Parse the command line.
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-v') THEN
      VERBOSE = .TRUE.
   ELSE IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: riq_intercept {options} inputfilename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Perform intercept analysis on VIPIR passive data'
      WRITE(*,*)
      WRITE(*,*)' -n1 First Number'
      WRITE(*,*)' -n2 Second Number'' '
      WRITE(*,*)' -r1 Lower range for statistics. Default=30 [km]'
      WRITE(*,*)' -r2 Upper range for statistics. Default=maximum [km]'
      WRITE(*,*)' -r Raw plot of intercepts between N1 and N2. Ignores shuffle settings'
      WRITE(*,*)' -I Enable Simple Phase Interferometry (SPID), enabled by default'
      WRITE(*,*)' -S {n} Shuffle-mode data with {n} interleaves'
   ELSE IF (CLP.EQ.'-r1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-I') THEN
      SIMODE=.TRUE.   
   ELSE IF (CLP.EQ.'-r') THEN
      RAW=.TRUE.
      IF(N1.EQ.0) N1=1
      IF(N2.EQ.0) N2=1000
   ELSE IF (CLP.EQ.'-S') THEN
      SHUFFLEMODE=.TRUE.
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) NUMSHUFFLE
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-d') THEN
      DEBUG=.TRUE.
   ELSE IF  (CLP.EQ.'-ba') THEN
      BLANKA=.TRUE.
   ELSE IF  (CLP.EQ.'-bp') THEN
      BLANKP=.TRUE.
   ELSE IF  (CLP.EQ.'-bf') THEN
      BLANKF=.TRUE.
   ELSE 
!    This is the input file name
     INFILE=CLP
   ENDIF
ENDDO
!
IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start run',TimeHack,CTime 
ENDIF
!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(*,*) ' readriq - error - Missing input file'
   CALL EXIT(-1)
ENDIF
!
! Zero out the ionogram (PRI) structures.
DO IPRI=1,MAXPRI
   CALL ZeroPRI(PRI(IPRI))
ENDDO
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             G95 has this feature, and in October 2010 gfortran version 4.4.5 also does.
!             Your mileage will vary. 
!-----------------------------------------------------------------------------------------
!
OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
CALL FixSCTstrings(SCT)
IF(VERBOSE) CALL DumpSCT(SCT)
!
! Todo:  Call an updatesct() type function that fixes the SCT based on local data
! This will be critical because most antenna configurations in most SCT's are WRONG!!!
!
! Set variables based on command line parameters
!
! Set Range Gate Limits.
RGT1=15          ! To avoid Tx and Rx impulse response 
RGT2=SCT%timing%gate_count
IF(RANGE1.GT.0) THEN
   RGT1=NINT((RANGE1-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT1=MIN(MAX(1,RGT1),SCT%timing%gate_count)
ENDIF
IF (RANGE2.GT.0) THEN
   RGT2=NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=MIN(MAX(1,RGT2),SCT%timing%gate_count)
ELSE IF (RANGE2.LT.0) THEN
   RGT2=-NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=SCT%timing%gate_count-RGT2
ENDIF
!
! Set amplitude limits
UNDERFLOW=1.0
IF (SCT%receiver%word_format .EQ.2) UNDERFLOW=0.001
!
! Get the processing time
CALL FDATE(TimeStamp)

! Set ShuffleMode based on the flags and SCT
! NOTE:  In general there will be more than one fixed frequency, and this can not be 
!        easily determined from the SCT, so the -S {numshuffle} command is needed.
! 
SHUFFLEMODE= (SHUFFLEMODE.OR.(SCT%Frequency%tune_type.EQ.4).OR.(SCT%Frequency%tune_type.EQ.5))
IF ((SHUFFLEMODE).AND.(NUMSHUFFLE.EQ.0)) THEN
   NUMSHUFFLE = 5    ! A guess
ENDIF
!
NUMFIXED=NUMSHUFFLE-1
! Modes which contain only fixed frequency data are not shuffle modes. 
! They are multiple fixed frequency modes 
IF (NUMSHUFFLE.LT.1) THEN
!  ToDo:  Determine the number of fixed frequencies from the SCT
   NUMFIXED=4
ENDIF
! From this point on, the NUMFIXED will be used instead of NUMSHUFFLE.
! The shuffle mode exception will be to skip the sweep frequency PRIs
!
IF (DEBUG) WRITE(*,*) 'SHUFFLEMODE= ',SHUFFLEMODE,' NUMSHUFFLE= ',NUMSHUFFLE, ' NUMFIXED= ', NUMFIXED  
!
! Determine the decode sequence
! ToDo:  Support Ramps and Repeats.

IF(DEBUG) THEN
   CALL FDATE(TimeHack)
   CALL CPU_TIME(CTime)
   WRITE(*,*) 'Start Read Loop ',TimeHack, CTime
ENDIF

EOF=.FALSE.
CTime=0.0
IFIXED=0
NUMPRI=0
NUMICPT=0
!
! Determine the pulse set length
NPRIBLOCK=SCT%frequency%pulse_count
IF (DEBUG) WRITE(*,*) 'NPRIBLOCK= ',NPRIBLOCK
!
! Output Headers
!
   WRITE(*,102) '#','NumAve','IPRI','NI','IS','Year Day     Time','UT_Time','Freq','Zn','ZnVar','Az','AzVar','Dop','DopVar', &
                'Kx','Ky','Kz','dKx','dKy','dKz','aKx','aKy','aKz', 'aKxerr','aKyerr','aKzerr'
!
!
! For the first go-around, use RTI mode (PRI-by-PRI) instead of pulset based processing.
! In almost all cases, there will be discontunities at each PRI.
DO WHILE (.NOT.EOF)           ! This is the main processing loop.
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
!
! There will be a loop here to loop over PRIs in a pulset.  OR not...
! The concept the the Intercept() contains data for all shuffled sequences changes the need for skipping much.
   SKIPPRI=.TRUE.    ! Read at least one PRI
   DO WHILE (SKIPPRI.AND.(.NOT.EOF))
      NUMPRI=NUMPRI+1
!      IPRI=NUMPRI
      IPRI=1    !  Hack, single PRI processing if we ignore pulsets, ramps, repeats
      CALL ZeroPCT(PCT)
      EOF=READPCT(IU,PCT)
      IF(EOF) CYCLE
      CALL FixPCTstrings(PCT)
      IF(VERBOSE) CALL DumpPCT(PCT)
!     Copy relevant elements from SCT and PCT to the PRI Structure
      CALL SCTPCT2PRI(SCT,PCT, PRI(IPRI))
      EOF=READPRI(IU,FLIPHASE,SCT%receiver%word_format,   PRI(IPRI))         ! Read one A-scan
      IF(EOF) CYCLE
      ISHUFFLE = MOD((NUMPRI-1),NUMSHUFFLE) + 1
      SKIPPRI=(RAW.AND.(NUMPRI.LT.N1))                               ! 17Feb15 Speeds up skipping
      IF (DEBUG) THEN
         WRITE(*,*) ' NUMPRI= ',NUMPRI, 'SKIPPRI= ',SKIPPRI,' ISHUFFLE= ',ISHUFFLE
         WRITE(*,*) ' IPRI= ', IPRI, 'PRI%Frequency = ',PRI(IPRI) %Frequency
      ENDIF 
   ENDDO   ! Skipping PRI's 

!     Optional signal calibration
   IF (RXCAL) CALL CALIBRATE(SCT%station%ursi_id, SCT%start_epoch, CALMETHOD, MASKRX,   PRI(IPRI))
!
! Process the PRI for intercept data (angle of arrival and Doppler)
!
!  Compute Amplitude and Phase from I/Q values
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Start compute amplitude and phase '
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
   CALL AMPHASE(DEBUG, PRI(IPRI))
!
!  Optional Angles and Doppler
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Start Interferometry = ', SIMODE
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
!
   NPRI=NUMPRI    !  Again a hack.  Need to resolve NUMPRI,IPRI,NPRI confusion.
!
   IF (SIMODE) THEN
      CALL SPID(DEBUG,SCT,PCT,   PRI(IPRI))
   ELSE
      WRITE(*,*) "You have asked for a non-existing processing method"
      CALL EXIT (-1)
   ENDIF
!
!  Raw data output.  For this program, its not I/Q (use riq_ionogram for that!)
!  This is raw AoA and Doppler for each PRI
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Start Raw Print = ', RAW,' NPRI=',NPRI,' N1=',N1,' N2=',N2
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF
   IF (RAW.AND.(NPRI.GE.N1).AND.(NPRI.LE.N2)) THEN
!     Raw dump of Interferometry data
      WRITE(*,112) 'IPRI','IRNG','UT_Time','dt','Freq','Range','Kx','Ky','Kz', 'Kxerr','Kyerr','Kzerr',&
                    'aKx','aKy','aKz', 'aKxerr','aKyerr','aKzerr','Zenith','Azimuth','ZnErr','AzErr','Pox','Poy','Rx','Ry' 
!     Start migrating raw output to a format similar to ionogram data
      TK=PRI(IPRI)%TK
      DO IRNG=1,PRI(IPRI)%GateCount
         WRITE(*,113) NPRI, IRNG, PRI(IPRI)%UTtime+1.0E-6*PRI(IPRI)%RGTime(IRNG), PRI(IPRI)%RGTime(IRNG), &
             PRI(IPRI)%Frequency/1000.0, 150./1000.*PRI(IPRI)%RGTime(IRNG), (PRI(IPRI)%VK(IRNG,I),I=1,3), &
             (PRI(IPRI)%VkErr(IRNG,I),I=1,3),(PRI(IPRI)%VK(IRNG,I)/TK,I=1,3), (PRI(IPRI)%VkErr(IRNG,I)/TK,I=1,3), &
              DpR*PRI(IPRI)%Zenith(IRNG),DpR*PRI(IPRI)%Azimuth(IRNG),DpR*PRI(IPRI)%ZnErr(IRNG),DpR*PRI(IPRI)%AzErr(IRNG), &
             (DpR*PRI(IPRI)%Phase0(IRNG,I),I=1,2), (PRI(IPRI)%CorrC(IRNG,I),I=1,2)
      ENDDO
      IF(BLANKA) WRITE(*,*)
      IF(BLANKF) WRITE(*,*)
   ENDIF

!  Exit if raw modes and we are past N2
   EOF = EOF.OR.(RAW.AND.(NUMPRI.GE.N2))               ! 17Feb15 Speeds up skipping

112 FORMAT ('#',A5,A6,1X,A14,A8,A12,A8,  12A10,  16A10)
113 FORMAT (2I6,1X,F14.6,F8.1,F12.5,F8.1,3F10.4,3F10.6,3F10.4,3F10.6,2F10.2,2F10.2, 2F10.4, 10F10.2)
!
!  Process this PRI into the Intercept
!
!  Determine which slot.
   IF(SWEEP.AND.ODD(NPRI))THEN
!     The sweep data are in the odd PRI's, they go into slot 1, which is reserved for sweeps
      IS=1
      NI=INT(NPRI/2) + 1
   ELSE
!     It's a fixed frequency
      IS=MOD(INT((NPRI-1)/(NPRIBLOCK*(NUMSWEEP+1))), NUMFIXED) + 2  ! A tortured pair of formulae, sorry :( 
      NI=NPRIBLOCK*INT( (NPRI-1) / (NPRIBLOCK*NUMFIXED*2*NUMSWEEP) ) + 1 + MOD(INT((NPRI-1)/2), NUMFIXED ) 
   ENDIF
!  Or do it this way
   NUMINTERCEPT(IS)=NUMINTERCEPT(IS)+1
!      
!
   IF (ISHUFFLE.EQ.1) NUMICPT = NUMICPT + 1
   IF(DEBUG) THEN
      CALL FDATE(TimeHack)
      CALL CPU_TIME(CTime)
      WRITE(*,*) 'Start PRI processing  NPRI=',NPRI,' IPRI=',IPRI,' ISHUFFLE=',ISHUFFLE,' NUMICPT=',NUMICPT
      WRITE(*,*) 'Time= ',TimeHack, 'CPUTIME= ',CTime,'NFRQ= ',NFRQ
   ENDIF

   STATMODE=0  !  Only one defined for now, Centeral Moments
!
!  Initiliaze the Intercept structure
   CALL InitIntercept(SCT, URSICODE, Intercept(IS,NI))
   Intercept(IS,NI)%Statistic=STATMODE
   Intercept(IS,NI)%Method="Carrier Phase Analysis"
   Intercept(IS,NI)%RG1=RGT1
   Intercept(IS,NI)%RG2=RGT2

   Intercept(IS,NI)%Statistic=STATMODE
   Intercept(IS,NI)%Statistic=STATMODE
   
   CALL PRI2Intercept(DEBUG,RMIN,PRI,  Intercept(IS,NI),PRISTats)     !  This does all the real work
   IF(DEBUG) WRITE(*,103) NPRI,IPRI,NI,IS, Intercept(IS,NI)%TimeString, Intercept(IS,NI)%UTtime, Intercept(IS,NI)%Frequency, &
                DpR*Intercept(IS,NI)%Zenith,DpR*Intercept(IS,NI)%ZenithVar,&
                DpR*Intercept(IS,NI)%Azimuth,DpR*Intercept(IS,NI)%AzimuthVar, &
                Intercept(IS,NI)%Doppler,Intercept(IS,NI)%DopVar,Intercept(IS,NI)%VK,Intercept(IS,NI)%VkVar


ENDDO  ! Main processing loop
!
! Sorted Output Section
! Placing the data into the structure allows for selective and sorted output at the end of the processing loop
DO IS=2,NUMSHUFFLE
   DO NI=1,NUMINTERCEPT(IS)
        TK=Intercept(IS,NI)%TK
        WRITE(*,103) Intercept(IS,NI)%NumAve,IPRI,NI,IS, Intercept(IS,NI)%TimeString, Intercept(IS,NI)%UTtime, &
                Intercept(IS,NI)%Frequency, DpR*Intercept(IS,NI)%Zenith,DpR*Intercept(IS,NI)%ZenithVar,&
                DpR*Intercept(IS,NI)%Azimuth,DpR*Intercept(IS,NI)%AzimuthVar, Intercept(IS,NI)%Doppler, &
                Intercept(IS,NI)%DopVar,Intercept(IS,NI)%VK,Intercept(IS,NI)%VkVar, &
                Intercept(IS,NI)%VK/TK,Intercept(IS,NI)%VkVar/TK
   ENDDO
   IF(BLANKA) WRITE(*,*)
   IF(BLANKF) WRITE(*,*)
ENDDO
 

102 FORMAT(A1,A5,3A6,3X,A17,2A12,30A10)
103 FORMAT(1X,I5,3I6,3X,A17,F12.3,F12.5,6F10.2,20F10.5)
END PROGRAM riq_intercept
