SUBROUTINE SPID(DEBUG,SCT,PCT,  PRI)
!
! Simple Phase Interferometer.
!
! Compute zenith and azimuth angles for each range gate in the PRI using 
! simple interferometry on the received phases.
! This method treats each PRI independently.
!
! Author: Tery Bullett,   University of Colorado CIRES
! 10 August 2011
! Linux using gfortran compiler
!
! License:
! This software is the intellectual property of the author.
! Please respect this.
! Generous terms of use are likely if you ask nicely.
!
! Purpose:
! "To give glory to God by exploring His universe with reverence"  
!
! History:
! 16Feb15 - Re-start work on this routine.  It has been undone for much too long
!
!
! Requires:
!    LINREG  Linear regression subroutine
!    XYZ2RTP Convert cartesian to spherical
!
!
! Point-Slope line formula
!             (Y2 - Y1)  
!     Y  =   ----------- [ X - X1 ]   +  Y1
!             (X2 - X1)
!
!   In our case, Phase is Y and Rx position is X

USE misc_utils
IMPLICIT NONE
INCLUDE 'type-param.f90'! Global Types and parameter definitions 
INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI
!
! 
! Arguments
LOGICAL,INTENT(IN):: DEBUG
TYPE(SCTtype),INTENT(IN) :: SCT
TYPE(PCTtype),INTENT(IN) :: PCT 
TYPE(PRItype),INTENT(INOUT) :: PRI

! Local Variables
! MapA - Antenna Map is the mapping of receiver number to the sequence needed for this algorithm.
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2 = De-aliasing pairs ; 3,4 are the outer antennas
! MapD - Distance Map is the mapping of antennas by increasing value of x or y
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2,3,4 Antenna number in increading values of x or y 
INTEGER,DIMENSION(2,4):: MapA,MapD ! Antenna Mapping, per above 
INTEGER,DIMENSION(8):: N2PI,N2PID   !  The number of 2pi phase aliases for each receiver/antenna
REAL,DIMENSION(8):: RawPhase, FixPhase 
INTEGER I,J,IRX,IRG,IXY,I2PI
INTEGER(KIND=1)::FLAG
REAL:: DPH,QF3,QF4,QF0,R2PI,TEMP
REAL,DIMENSION(4)::X,Y,XD,YD
REAL,DIMENSION(2):: P0,DP,R,P0D,DPD,RD  ! For the linear fit
LOGICAL:: PROBLEM,DDBUG ! Double Debug ;)
!   For angle of arrival calculations
REAL::WLEN,TK,THETA,PHI,RMIN
REAL,DIMENSION(3):: VK(3),AK(3),AKS(3)   ! K vectors 
!
!
! Antenna Geometry, for dirction finding
! MapA - Antenna Map is the mapping of receiver number to the sequence needed for this algorithm.
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2 = De-aliasing pairs ; 3,4 are the outer antennas
! MapD - Distance Map is the mapping of antennas by increasing value of x or y
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2,3,4 Antenna number in increading values of x or y 
!
! This is hard-wired for Wallops.  An obvious ToDo
! Miniumal use is made of the PRI%Flag.  Much refinement is possible.
!
DATA (MapA(1,I),I=1, 4) /6,8,2,4/
DATA (MapA(2,I),I=1, 4) /5,7,1,3/
DATA (MapD(1,I),I=1, 4) /4,8,6,2/
DATA (MapD(2,I),I=1, 4) /3,7,5,1/
!
! This is the threshold quality factor for R^2 on the linear fit.
! Values higher than this will cause a test on multiple values of N2PI
QF0 = 0.38
!
!
PRI%Zenith=0.0
PRI%Azimuth=0.0
PRI%Doppler=0.0
PRI%ZnErr=1.0
PRI%AzErr=1.0
PRI%DopErr=1.0
PRI%Flag=0
DDBUG=.TRUE.
DDBUG=.FALSE.
!
IF (DEBUG) WRITE(*,*) "SPID"
IF (DEBUG) WRITE(*,100) '#','IRG','|K|','Kx','Ky','Kz','ax','ay','az','Theta','Phi','Pox','Poy','Rx','Ry'  
!
! Some values for the PRI
TK=1.0
WLEN = c/(1000.0*PRI%Frequency)  ! Wavelength in meters, Frequency is in kHz
IF(WLEN.NE.0.0) TK = TWOPI/WLEN   ! Total K magnitude
PRI%TK=TK
!
! Fill the RawPhase() array from the PRI
DO IRG=1,PRI%GateCount
!
   DO IRX = 1, PRI%ReceiverCount
      RawPhase(IRX)=PRI%Phase(IRX,IRG)
   ENDDO
!
!  The De-aliasing process fills the array N2PI() for each receiver/antenna.  The phase to be used in calculations is 
!  the measured phase minus N2PI(i)*TWOPI for each antenna (i).
!
!  De-alias the phases
   N2PI=0
   P0=0.0
   DP=0.0
   R=0.0
   P0D=0.0
   DPD=0.0
   RD=0.0
   PROBLEM=.FALSE.
!
   DO IXY=1,2       ! Each X and Y directions independent.  This may cause different phase offests at the origin?
      DPH = PRI%Phase(MapA(IXY,2),IRG) -  PRI%Phase(MapA(IXY,1),IRG)
!     This method defines the first antenna to be un-aliased.   Consequences?
      IF (DPH.GT.PI) THEN
         N2PI(MapA(IXY,2)) =  1
      ELSE IF (DPH.LT.-PI) THEN
         N2PI(MapA(IXY,2)) = -1
      ELSE
         N2PI(MapA(IXY,2)) = 0
      ENDIF
      DPH = DPH - TWOPI*N2PI(MapA(IXY,2))
!     Fit a phase slope line using these unwrapped phases, using the point-slope formula for a line.
!     The point is the definded-to-be unaliased first receiver in the inner pair
      X(1) = SCT%Station%rx_position(IXY,MapA(IXY,1))
      X(2) = SCT%Station%rx_position(IXY,MapA(IXY,2))
      Y(1) = PRI%Phase(MapA(IXY,1),IRG) - TWOPI*N2PI(MapA(IXY,1))
      Y(2) = PRI%Phase(MapA(IXY,2),IRG) - TWOPI*N2PI(MapA(IXY,2))
      DP(IXY)=1.0E6
      IF (X(1).NE.X(2))  DP(IXY) = (Y(2)-Y(1))/(X(2)-X(1))    ! Phase slope.
      P0(IXY) = DP(IXY)*(-X(1)) + Y(1)                ! Phase at the center of the array (X,Y=0)
      IRX=MapA(IXY,1)
      IF (DDBUG) WRITE(*,201) 'RG','XY','Rx','N2Pi','Amp','Phase','X','Y','DPH','P0','DP','QF','R'
      IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(1),DpR*Y(1),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY)
      IRX=MapA(IXY,2)
      IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(2),DpR*Y(2),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY)
!     De-alias third in the sequence
      X(3) = SCT%Station%rx_position(IXY,MapA(IXY,3))
      Y(3) = DP(IXY)*(X(3)-X(1)) + Y(1)        ! Predicted phase based on linear fit
      DPH = PRI%Phase(MapA(IXY,3),IRG) - Y(3)  ! Difference between prediction and actual
!db      write(*,*) x(3), X(1), Y(1),Y(3), DP(IXY), PRI%Phase(MapA(IXY,3),IRG), DPH, DpR
      R2PI = DPH/TWOPI          ! Number of aliased rotations 
      I2PI = NINT(R2PI)         ! Fractional rotations
      N2PI(MapA(IXY,3)) = I2PI  ! Number of 2Pi wraps
      QF3 = SIGN( ABS(R2PI)-ABS(I2PI), R2PI ) ! Questionable Factor
!db  write(*,*) R2PI,I2PI,QF3 
      Y(3) = PRI%Phase(MapA(IXY,3),IRG) - TWOPI*N2PI(MapA(IXY,3))  ! De-aliased Phase
      IRX=MapA(IXY,3)
      IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(3),DpR*Y(3),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF3,R(IXY)
!
!     De-alias fourth in the sequence
      X(4) = SCT%Station%rx_position(IXY,MapA(IXY,4))
      Y(4) = DP(IXY)*(X(4)-X(1)) + Y(1)        ! Predicted phase based on linear fit
      DPH = PRI%Phase(MapA(IXY,4),IRG) - Y(4)  ! Difference between prediction and actual
      R2PI = DPH/TWOPI          ! Number of aliased rotations 
      I2PI = NINT(R2PI)         ! Fractional rotations
      N2PI(MapA(IXY,4)) = I2PI  ! Number of 2Pi wraps
      QF4 = SIGN( ABS(R2PI)-ABS(I2PI), R2PI ) ! Questionable Factor
      Y(4) = PRI%Phase(MapA(IXY,4),IRG) - TWOPI*N2PI(MapA(IXY,4))
! 
!     Finished initial de-aliasing the phases
!     Fit a line to each of the 4 points, detmine slope, Y-intercept, and fit value
      CALL LINREG(4,X,Y,  DP(IXY),P0(IXY),R(IXY))
      IRX=MapA(IXY,4)
      IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(4),DpR*Y(4),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF4,R(IXY)
!
!     Sometimes, the phases of (3) or (4) are close to 180 deg from the prediction and the choise to 2Pi unwrap or not
!     is not obvious, using only the inner antennas.  However, one choice is slightly better than the other
!
      RD=R               ! Save the old value of R
      P0D=P0             ! and Phase0
      DPD=DP             ! and Phase Slope
      N2PID=N2PI         ! and N2PI, in case we go back
      XD=X
      YD=Y
!
      IF ((ABS(QF3).GT.QF0).AND.(ABS(QF4).GT.QF0)) THEN
!        Both outer phase fits are bad.  Suck it up, buttercup and use R to reject the data
         CONTINUE
      ELSE IF ((ABS(QF3).GE.QF0)) THEN
!        Try the adjcent N2PI value on #3 and see if it helps
!x         write(*,*) 'old n2pi: ',N2PI(MapA(IXY,3))
         N2PI(MapA(IXY,3)) = N2PI(MapA(IXY,3)) + INT(SIGN(1.0, QF3))       ! Add one if QF>0, subtract if QF<0 
!x         write(*,*) 'new n2pi: ',N2PI(MapA(IXY,3))
!x         write(*,*) 'old Y: ',Y(3)
         Y(3) = PRI%Phase(MapA(IXY,3),IRG) - TWOPI*N2PI(MapA(IXY,3))  ! Trial De-aliased Phase
!x         write(*,*) 'new Y: ',Y(3)
!x         write(*,*) 'Old: ' ,DP,P0,R
         CALL LINREG(4,X,Y,  DP(IXY),P0(IXY),R(IXY))
!x         write(*,*) 'New ' ,DP,P0,R
         IRX=MapA(IXY,3)
         IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(3),DpR*Y(3),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF3,R(IXY)
         IF (ABS(RD(IXY)).GE.ABS(R(IXY))) THEN       ! This was not a good move, go back
            R=RD
            P0=P0D
            DP=DPD
            N2PI=N2PID
            Y=YD
         ENDIF
         IRX=MapA(IXY,3)
         IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(3),DpR*Y(3),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF3,R(IXY)
!
      ELSE IF ((ABS(QF4).GE.QF0)) THEN
!        Try the adjcent N2PI value on #4 and see if it helps
         N2PI(MapA(IXY,4)) = N2PI(MapA(IXY,4)) + INT(SIGN(1.0, QF4))       ! Add one if QF>0, subtract if QF<0 
         Y(4) = PRI%Phase(MapA(IXY,4),IRG) - TWOPI*N2PI(MapA(IXY,4))  ! Trial De-aliased Phase
         CALL LINREG(4,X,Y,  DP(IXY),P0(IXY),R(IXY))
         IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(4),DpR*Y(4),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF4,R(IXY)
         IF (ABS(RD(IXY)).GE.ABS(R(IXY))) THEN       ! This was not a good move, go back
            R=RD
            P0=P0D
            DP=DPD
            N2PI=N2PID
            Y=YD
         ENDIF
         IF (DDBUG) WRITE(*,200) IRG,IXY,IRX,N2PI(IRX),PRI%AmpdB(IRX,IRG),DpR*(PRI%Phase(IRX,IRG)), &
                              X(4),DpR*Y(4),DpR*DPH,DpR*P0(IXY),DpR*DP(IXY),QF4,R(IXY)
      ENDIF
!
      IRX=MapA(IXY,4)
!
!     Save the Dealiased phase data.  Why??
      FixPhase(MapA(IXY,1)) = Y(1)
      FixPhase(MapA(IXY,2)) = Y(2)
      FixPhase(MapA(IXY,4)) = Y(4)
      FixPhase(MapA(IXY,3)) = Y(3)
! 
   ENDDO   ! X and Y directions
!
   
   IF (DEBUG) THEN
      WRITE(*,105) '#','RG','Frq',('XY','DP','P0','R', J=1,2),'dP0'
      WRITE(*,101) '#', IRG,PRI%Frequency, (J, DpR*DP(J),DpR*P0(J),R(J), J=1,2),DpR*(P0(2)-P0(1))
      WRITE(*,105) '#',('Rx','X','Raw','Fix','N2PI',J=1,2)
      WRITE(*,102) ((MapD(J,I),SCT%Station%rx_position(J,MapD(J,I)),DpR*RawPhase(MapD(J,I)),DpR*FixPhase(MapD(J,I)), &
                         N2PI(MapD(J,I)), J=1,2),I=1,2)
      WRITE(*,103) (0,0.0,DPR*DP(J),DPR*P0(J),R(J),J=1,2) !      The center of the array
      WRITE(*,102) ((MapD(J,I),SCT%Station%rx_position(J,MapD(J,I)),DpR*RawPhase(MapD(J,I)),DpR*FixPhase(MapD(J,I)), &
                         N2PI(MapD(J,I)), J=1,2),I=3,4)
      WRITE(*,*)
!      WRITE(*,*)

   ENDIF   ! Debug
!
!  Convert the phase slopes into K vector components and angles of arrival.
!  Use the relationship that the projection of the K vector in the X and Y directions is directly proportional to the phase slope 
!  or dP/dx and dP/dy, and |K| is 2*Pi/Lambda always.  So |ax| = Lamba/2*pi * DPx and same for Y
   VK(1) = DP(1)    ! X component of K
   VK(2) = DP(2)    ! Y component of K
   TEMP = TK**2 - VK(1)**2 -VK(2)**2
!
   IF(TEMP.GE.0.0) THEN 
      VK(3) = SQRT(TEMP)  !  Z component of K for everything right with the world
   ELSE IF (ABS(TEMP).GT.0.05*TK) THEN
      VK(3) = 0.0         !  There is a 'small' problem with the numbers not adding up right, let it slide
   ELSE
      VK(3) = 0.0         !  An obvious error in Kx or Ky.  Flag this as bad
      FLAG=-1
   ENDIF
   AK=VK/TK   ! Matrix divided by a scalar seems to work
   THETA=0.0
   PHI=0.0
   CALL XYZ2RTP(AK, AKS) ! Convert unit vector AK to spherical AKS
   THETA=AKS(2)
   PHI=AKS(3)
!
!  Add this to the PRI structure
   PRI%Zenith(IRG) = THETA
   PRI%Azimuth(IRG) = PHI
   PRI%VK(IRG,1) = VK(1)
   PRI%VK(IRG,2) = VK(2)
   PRI%VK(IRG,3) = VK(3)
!
   PRI%Flag(IRG)=FLAG   
!
   PRI%Phase0(IRG,1) = P0(1)
   PRI%Phase0(IRG,2) = P0(2)
   PRI%CorrC(IRG,1) = R(1)
   PRI%CorrC(IRG,2) = R(2)
!
! Use the Correlation Coefficient R to estimate a error bar.  Placing a magnitude on this can be tricky.
! Start with a WAG.  Let's let the error bar minimize to about 5 degree of phase for perfect R=1 and then
! use 1/(R^4) as an error estimate.  Note this is really R^2.  What to do with the Z component?  Set to 5 degree
! This is incredibly unscientific.  It just fills a box  
!
!  This value of RMIN sets the maximum error bar.  Must be > 0
   RMIN=0.2
!
   IF (PROBLEM) THEN
      PRI%VKErr(IRG,1)=0.2*TK   !  Set a default large error that will hopefully get noticed
      PRI%VKErr(IRG,2)=0.2*TK
      PRI%VKErr(IRG,3)=0.2*TK
      PRI%ZnErr(IRG)=0.1*PI
      PRI%AzErr(IRG)=0.1*PI
   ELSE
      PRI%VKErr(IRG,1)=TK/(72.0*MAX(R(1),RMIN))
      PRI%VKErr(IRG,2)=TK/(72.0*MAX(R(2),RMIN))
      PRI%VKErr(IRG,3)=TK/(72.0)
!     ToDo:Properly map into spherical coordinates.  Remember, angles in radians here
      PRI%ZnErr(IRG)=10.0*MAX(PRI%VKErr(IRG,1),PRI%VKErr(IRG,2))
      PRI%AzErr(IRG)=PRI%ZnErr(IRG) 
   ENDIF
!
!
   IF(DEBUG) WRITE(*,104) IRG, TK,VK, AK, DpR*THETA, DpR*PHI, DpR*P0, R 

ENDDO  ! Range Gates

! All Done
RETURN

100 FORMAT (A1,A5,20A8)
101 FORMAT (A1,I5,F8.1,I8,2F8.1,F8.4,I8,2F8.1,F8.4,F8.1)
102 FORMAT (1X,I5,3F8.1,I8,I8,3F8.1,I8)
103 FORMAT (1X,I5,3F8.1,F8.3, I8,3F8.1,F8.3)
104 FORMAT (1X,I5,F8.3,3F8.3,3F8.3,4F8.1,2F8.3)
105 FORMAT (A1,A5,12A8)
200 FORMAT (I5,I3,2I8,7F8.1,3F8.3)
201 FORMAT (A5,A3,12A8)
END SUBROUTINE SPID
